from tkinter import *
from tkinter import ttk
from tkinter.filedialog import askopenfilename

import os

import numpy as np

import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg

from PIL import Image, ImageTk

from ripser import ripser
from persim import plot_diagrams


def fileselect():
    filename = askopenfilename()
    ext = os.path.splitext(filename)[-1].lower()
    if (ext == ".jpg"):
        
        plt.clf()
        
        img = Image.open(filename)
        
        arr = np.array(img)
        arr_ravel = np.ravel(arr)

        r = arr_ravel[::3]
        g = arr_ravel[1::3]
        b = arr_ravel[2::3]
        
        global spinval
       
        points = 250
        step = int(np.ceil(len(r)/points))

        r = r[::step]
        g = g[::step]
        b = b[::step]

        rgb = (r, g, b)
        rgb = np.transpose(rgb)
        
        global fig
        
        #image
        ax = fig.add_subplot(2, 2, 1)
        ax.set_title("Image")
        ax.get_xaxis().set_visible(False)
        ax.get_yaxis().set_visible(False)
        ax.imshow(arr)
        
        #color space
        ax = fig.add_subplot(2, 2, 2, projection="3d")

        ax.set_xlim3d(0, 255)
        ax.set_ylim3d(0, 255)
        ax.set_zlim3d(0, 255)

        ax.set_xlabel('Red')
        ax.set_ylabel('Green')
        ax.set_zlabel('Blue')

        ax.set_title("Color Space")
        
        #rgb normalized to [0,1]
        ax.scatter3D(r, g, b, c=rgb/255)
        
        #persistence diagrams
        ax = fig.add_subplot(2, 2, 3)
        ax.set_title("Birth/Death")
        diagrams = ripser(rgb, maxdim=2)['dgms']
        plot_diagrams(diagrams, show=False, ax=ax)
        
        ax = fig.add_subplot(2, 2, 4)
        ax.set_title("Lifetime")
        plot_diagrams(diagrams, show=False, ax=ax, lifetime=True)
        
        global canvas
        canvas.draw()
        #plt.savefig(filename[:-4] + "_color_space.png")
    else:
        pass

#matplotlib likes to stick around when the root is destroyed, quitting then destroying manages that    
def quit():
    root.quit()
    root.destroy()
    

root = Tk()
root.title("Color Space")
root.resizable(False, False)

mainframe = ttk.Frame(root, padding="3 3 3 3")
mainframe.grid(column=0, row=0, sticky=(N, W, E, S))
root.columnconfigure(0, weight=1)
root.rowconfigure(0, weight=1)

fig = plt.figure(figsize=(8,8))
canvas = FigureCanvasTkAgg(fig, mainframe)
canvas.get_tk_widget().grid(row=0)

ttk.Button(mainframe, text="Select a .jpg to find its color space", command=fileselect).grid(row=1)

root.protocol("WM_DELETE_WINDOW", quit)
root.mainloop()